/* File:    Blinky_gcc.c
 * 
 * Author:  David Zemon
 * Project: Blinky_gcc
 */

// Includes
#include "Blinky_gcc.h"

// Constants
extern uint32 *_load_start_cogblinky;

// Main function
void main (void) {
	uint32 secondCog = 0, thirdCog = 0;
	uint32 refresh = CLKFREQ >> 1;

	// Start two new cogs blinking LEDs, one with assembly, one with compiled C
	thirdCog = cognew(standalone_blinky, 0);
	secondCog = cognew(_load_start_cogblinky, 0);

	// If either one failed, run a blinky pattern in the current cog showing the error
	if ((uint32) -1 == secondCog)
		c_blinky(refresh / 8, secondCog << 16);
	if ((uint32) -1 == thirdCog)
		c_blinky(refresh / 8, thirdCog << 16);

	// Initialization complete, blink the cogID of the assembly cog
	c_blinky(refresh, secondCog << 16);
}

/* @brief	Blink an LED in the current cog
 *
 * @param	refresh		clock cycles between blinks
 * @param	cogID		number between 0 and 7 shifted 16 bits to the left
 */
void c_blinky (const uint32 refresh, const uint32 cogID) {
	uint8 i;
	GPIODirModeSet((LED_1), GPIO_DIR_OUT);

	while (1) {
		// Show which LEDs are being used (gives bit-count)
		GPIOPinWrite((LED_1), -1);
		waitcnt(refresh + CNT);
		GPIOPinWrite((LED_1), 0);
		waitcnt(refresh + CNT);

		// Show cog ID in binary form
		for (i = 0; i < 3; ++i) {
			GPIOPinWrite((LED_1), cogID);
			waitcnt(refresh/10 + CNT);
			GPIOPinWrite((LED_1), 0);
			waitcnt(refresh/10 + CNT);
		}

		// Reset
		GPIOPinWrite((LED_1), 0);
		waitcnt(refresh + CNT);
	}
}

/* @brief	Blink an LED in a stand-alone cog
 *
 */
void standalone_blinky (void) {
	GPIODirModeSet(BIT_22, GPIO_DIR_OUT);

	while (1) {
		GPIOPinToggle(BIT_22);
		waitcnt(CLKFREQ/4 + CNT);
	}
}
